@@ -1,10 +1,7 @@
 import inspect
 import json
-import os
-import re
-import traceback
+from collections.abc import AsyncIterable
 from pathlib import Path
-from posixpath import expanduser
 from typing import Dict, List
 
 from loguru import logger
@@ -12,7 +9,7 @@ from loguru import logger
 try:
     import aiohttp
     import docker
-    from quart import Quart
+    from quart import Quart, make_response
     from quart import request as input_request
     from requests_toolbelt import MultipartDecoder
 except:
@@ -22,7 +19,8 @@ except:
 
 from werkzeug.exceptions import BadRequest, RequestEntityTooLarge
 
-from .model import AudioRequest, StructuredTextRequest, TextRequest
+from .model import (AudioRequest, Failure, Progress, ResponseObject,
+                    StructuredTextRequest, TextRequest)
 from .utils.docker import COPY_FOLDER, DOCKERFILE, ENTRYPOINT_QUART
 
 
@@ -146,6 +144,9 @@ class QuartService:
         Main request processing logic - accepts a JSON request and returns a JSON response.
         """
         logger.info("Process request")
+        even_stream = True if "text/event-stream" in input_request.accept_mimetypes else False
+        logger.debug(f"Accept MimeTypes: {input_request.accept_mimetypes}")
+        logger.info(f"Accept even-stream: {even_stream}")
         if "application/json" in input_request.content_type:
             data = await input_request.get_json()
         elif "multipart/form-data" in input_request.content_type:
@@ -160,9 +161,9 @@ class QuartService:
                 elif "audio" in headers["Content-Type"]:
                     data["content"] = part.content
                 else:
-                    raise ValueError("Unknown Content-Type in multipart request")
+                    raise ProcessingError.UnsupportedType(str(headers["Content-Type"]))
         else:
-            raise ValueError(f"Content-Type [{input_request.content_type}] not implemented yet.")
+            raise ProcessingError.UnsupportedType(input_request.content_type)
 
         logger.debug(f"Data type: {data.get('type')}")
         if data.get("type") == "audio":
@@ -175,11 +176,42 @@ class QuartService:
             raise ProcessingError.InvalidRequest()
         logger.info(f"Call with the input: {request}")
         logger.info("Await for the coroutine...")
-        response = await self.process_request(request)
-        logger.info(f"Get response: {response}")
-        response = {"response": response.dict(by_alias=True)}
-        logger.info(f"Return: {response}")
-        return response
+        try:
+            response = await self.process_request(request)
+        except:
+            raise ProcessingError.InvalidRequest()
+        if isinstance(response, Failure):
+            logger.info(f"Get error message: {response}")
+            response = {"failure": response.dict(by_alias=True)}
+            logger.info(f"Return: {response}")
+        elif isinstance(response, ResponseObject):
+            logger.info(f"Get response: {response}")
+            response = {"response": response.dict(by_alias=True)}
+            logger.info(f"Return: {response}")
+            return response
+        elif isinstance(response, AsyncIterable):
+            logger.info(f"Get async iterable response")
+            if even_stream:
+                response = await make_response(
+                    self.generator_mapping(response), 200, {"Content-Type": "text/event-stream"}
+                )
+                # Quart will by default time-out long responses, may be necessary to disable that
+                # or at least set a longer timeout than usual
+                response.timeout = None
+                return response
+            else:
+                response = await self.get_response_from_generator(response)
+                logger.info(f"Get response: {response}")
+                if isinstance(response, ResponseObject):
+                    response = {"response": response.dict(by_alias=True)}
+                elif isinstance(response, Failure):
+                    response = {"failure": response.dict(by_alias=True)}
+                else:
+                    raise ProcessingError.InvalidRequest()
+                logger.info(f"Return: {response}")
+                return response
+        else:
+            raise ProcessingError.InvalidRequest()
 
     async def process_request(self, request):
         """
@@ -187,13 +219,22 @@ class QuartService:
         """
         if request.type == "text":
             logger.debug("Process text")
-            return await self.process_text(request)
+            try:
+                return await self.process_text(request)
+            except:
+                return self.process_text(request)
         if request.type == "structuredText":
             logger.debug("Process structured text")
-            return await self.process_structured_text(request)
+            try:
+                return await self.process_structured_text(request)
+            except:
+                return self.process_structured_text(request)
         if request.type == "audio":
             logger.debug("Process audio")
-            return await self.process_audio(request)
+            try:
+                return await self.process_audio(request)
+            except:
+                return self.process_audio(request)
         raise ProcessingError.InvalidRequest()
 
     async def process_text(self, request: TextRequest):
@@ -223,6 +264,67 @@ class QuartService:
         """
         raise ProcessingError.UnsupportedType()
 
+    @staticmethod
+    async def generator_mapping(generator):
+        end = False
+        try:
+            async for message in generator:
+                if end == True:
+                    logger.warning(
+                        (
+                            "The service has already returned a ResponseObject or Failure message but continue to return the following message:\n"
+                            f"{message.dict(by_alias=True)}\nThis message will be ignored and not returned to the user."
+                        )
+                    )
+                    continue
+                if isinstance(message, Failure):
+                    logger.info(f"Get failure: {message}")
+                    message = json.dumps({"failure": message.dict(by_alias=True)})
+                    end = True
+                elif isinstance(message, Progress):
+                    logger.info(f"Get progress: {message}")
+                    message = json.dumps({"progress": message.dict(by_alias=True)})
+                elif isinstance(message, ResponseObject):
+                    logger.info(f"Get response: {message}")
+                    message = json.dumps({"response": message.dict(by_alias=True)})
+                    end = True
+                yield f"data:{message}\r\n\r\n"
+        except:
+            message = json.dumps(
+                {
+                    "failure": Failure(
+                        errors=[
+                            {
+                                "code": "elg.internalError",
+                                "text": "Internal error during processing",
+                                "params": ["message"],
+                            }
+                        ]
+                    ).dict(by_alias=True)
+                }
+            )
+            yield f"data:{message}\r\n\r\n"
+
+    @staticmethod
+    async def get_response_from_generator(generator):
+        response = None
+        async for message in generator:
+            if response is not None:
+                logger.warning(
+                    (
+                        "The service has already returned a ResponseObject or Failure message but continue to return the following message:\n"
+                        f"{message.dict(by_alias=True)}\nThis message will be ignored and not returned to the user."
+                    )
+                )
+                continue
+            if isinstance(message, (ResponseObject, Failure)):
+                response = message
+        if response is None:
+            return Failure(
+                errors=[{"code": "elg.response.invalid", "text": "Invalid response message", "params": ["message"]}]
+            )
+        return response
+
     @classmethod
     def create_requirements(cls, requirements: List = [], path: str = None):
         """
@@ -241,12 +343,21 @@ class QuartService:
             f.write("\n".join(set(requirements)))
 
     @classmethod
-    def create_docker_files(cls, required_files: List = [], required_folders: List = [], path: str = None):
+    def create_docker_files(
+        cls,
+        required_files: List[str] = [],
+        required_folders: List[str] = [],
+        commands: List[str] = [],
+        base_image: str = "python:slim",
+        path: str = None,
+    ):
         """Class method to create the correct Dockerfile.
 
         Args:
-            required_files (List, optional): List of files needed for the service. Defaults to [].
-            required_folders (List, optional): List of folders needed for the service. Defaults to [].
+            required_files (List[str], optional): List of files needed for the service. Defaults to [].
+            required_folders (List[str], optional): List of folders needed for the service. Defaults to [].
+            commands (List[str], optional): List off additional commands to run in the Dockerfile. Defaults to [].
+            base_image (str, optional): Name of the base Docker image used in the Dockerfile. Defaults to 'python:slim'.
             path (str, optional): Path where to generate the file. Defaults to None.
         """
         if path == None:
@@ -257,6 +368,7 @@ class QuartService:
         required_folders = "\n".join(
             [COPY_FOLDER.format(folder_name=str(Path(folder))) for folder in required_folders]
         )
+        commands = ["RUN " + cmd for cmd in commands]
         service_script = path.name[:-3]  # to remove .py
         # The docker-entrypoint file is a Linux shell script so _must_ be
         # written with Unix-style line endings, even if the build is being done
@@ -264,7 +376,14 @@ class QuartService:
         with open(path.parent / "docker-entrypoint.sh", "wb") as f:
             f.write(ENTRYPOINT_QUART.format(service_script=service_script).encode("utf-8"))
         with open(path.parent / "Dockerfile", "w") as f:
-            f.write(DOCKERFILE.format(required_files=" ".join(required_files), required_folders=required_folders))
+            f.write(
+                DOCKERFILE.format(
+                    base_image=base_image,
+                    required_files=" ".join(required_files),
+                    required_folders=required_folders,
+                    commands="\n".join(commands),
+                )
+            )
 
     @classmethod
     def docker_build_image(cls, tag: str, pull: bool = True, path: str = None, **kwargs):
