@@ -470,6 +470,7 @@ class Service(Entity, NeedAuthentication):
         Returns:
             Union[Dict, str]: service response in JSON format or as a string if output_func returns a string.
         """
+        headers["Accept"] = "application/json"
         response = requests.post(
             self.elg_execution_location_sync, headers=headers, data=data, timeout=timeout, **kwargs
         )
@@ -501,6 +502,7 @@ class Service(Entity, NeedAuthentication):
         response = requests.get(uri, headers=headers)
         waiting_time = time.time()
         while response.ok and "progress" in response.json().keys():
+            print(f"Progress: {response.json()['progress']['percent']}%")
             sleep(1)
             response = requests.get(uri, headers=headers, timeout=timeout)
             if time.time() - waiting_time > (timeout if timeout is not None else float("inf")):
