@@ -5,7 +5,7 @@ ENTRYPOINT_QUART = """\
 #!/bin/sh\nexec /sbin/tini -- venv/bin/hypercorn --bind=0.0.0.0:8000 "$@" {service_script}:app\n"""
 
 DOCKERFILE = """\
-FROM python:slim
+FROM {base_image}
 
 # Install tini and create an unprivileged user
 ADD https://github.com/krallin/tini/releases/download/v0.19.0/tini /sbin/tini
@@ -19,6 +19,8 @@ USER elg:elg
 
 WORKDIR /elg
 
+{commands}
+
 # Create a Python virtual environment for the dependencies
 RUN python -m venv venv 
 RUN /elg/venv/bin/python -m pip install --upgrade pip
